--[[ CombatTime ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
  ["showCombatLogging"] = false;
  ["combatLoggingReminder"] = true;
  ["askIn_LFR"] = true;
  ["askIn_LFG"] = true;
  ["askIn_Raid"] = true;
  ["askIn_Party"] = true;
  ["continueEncounterWhenDead"] = true;
  ["useDBMPullDetection"] = true;
  ["showYardsRunInCombat"] = false;
  ["autoLogAfterDisconnect"] = true;
  ["logoutTimestamp"] = -1;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    {
      ["type"] = "Boolean";
      ["opt"] = "showCombatLogging";
      ["desc"] = "display combat logging state";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "combatLoggingReminder";
      ["desc"] = "remind me to enable logging when entering an instance";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "askIn_LFR";
      ["desc"] = "show message in: LFR raid";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "askIn_LFG";
      ["desc"] = "show message in: LFG party";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Scenario";
      ["desc"] = "show message in: Scenario";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Raid";
      ["desc"] = "show message in: normal raid";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "askIn_Party";
      ["desc"] = "show message in: normal party";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "autoLogAfterDisconnect";
      ["desc"] = "automatically enable combat logging after a disconnect or relog (upto 10 minutes)";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "continueEncounterWhenDead";
      ["desc"] = "continue to count when your character died but the encounter is still in progress";
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "useDBMPullDetection";
      ["desc"] = "also use DBM's pull detection to start a combat session";
    },
    {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    },
    {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    },
    {
      ["type"] = "Boolean";
      ["opt"] = "showYardsRunInCombat";
      ["desc"] = "display yards run in a combat session";
    },
  };
}
local function getOptionIndexByName(name)
  if misc and misc["GUI"] then
    for i, v in ipairs(misc["GUI"]) do
      if v["opt"] == name then
        return i
      end
    end
  end
  return 0
end

local M = FixR:NewModule("CombatTime", DefaultO, misc)

local frame, confirmationFrame = nil, nil
local frameEvents = {};
local DBMEventHandler = {};
local moving = false
local absorbAmount = 0
local setOptionBoolean, getInstanceType, instanceTypeString

local function initFrame()
  frame = CreateFrame("Frame", "FixR_CombatTimeFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  frame.timeSinceLastUpdate = 0
  frame.needsUpdate = false
  frame.inCombat = false
  frame.encounterInProgress = false
  frame.secondsInCombat = 0
  frame.movement_done = 0
  
  --CHAT_MSG_ADDON
  frame.COMprefix = "FixRCombatTime"
  frame.COMchannel = "RAID"
  frame.COMPlayerName = UnitName("player")
  frame.COMlastStatusRequestSent = -1
  local COMok = RegisterAddonMessagePrefix(frame.COMprefix)
  --FixR:Echo("CombatTime: Registering prefix... ", COMok and "|cffaaffaasuccessful" or "|cffff8888failed", ".")
  
  frame.combatLogging = LoggingCombat() and true or false
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("CombatTime")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.movement_resetMapScale = function()
    --/dump GetPlayerFacing()
    --/run local _,sx1,sy1,sx2,sy2=GetCurrentMapZone();local px,py=GetPlayerMapPosition("player");print(abs(sx2-sx1)*px,abs(sy2-sy1)*py)
    --/run local _,sx1,sy1,sx2,sy2=GetCurrentMapZone();print(abs(sx2-sx1),abs(sy2-sy1))
    --SetMapToCurrentZone();
    local n, a1, b1, c1, d1 = GetCurrentMapDungeonLevel()
    if not (a1 and b1 and c1 and d1) then
      n, a1, b1, c1, d1 = GetCurrentMapZone()
    end
    if not (a1 and b1 and c1 and d1) then
      frame.movement_mapScaleX = 0
      frame.movement_mapScaleY = 0
    else
      frame.movement_mapScaleX = c1 - a1
      frame.movement_mapScaleY = d1 - b1
    end
    local px, py = GetPlayerMapPosition("player")
    frame.movement_playerLastX = px
    frame.movement_playerLastY = py
  end
  
  frame.startEncounter = function()
    frame.encounterInProgress = true
    frame.secondsInCombat = 0
    frame.timeSinceLastUpdate = 0
    
    frame.movement_resetMapScale()
    frame.movement_done = 0
  end
  
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    if self.encounterInProgress then
      local px, py = GetPlayerMapPosition("player")
      self.movement_done = self.movement_done + math.sqrt(((px - self.movement_playerLastX)*self.movement_mapScaleX)^2 + ((py - self.movement_playerLastY)*self.movement_mapScaleY)^2)
      self.movement_playerLastX = px
      self.movement_playerLastY = py
    end
    
    while (self.encounterInProgress and self.timeSinceLastUpdate > 1) do --in seconds
      self.secondsInCombat = self.secondsInCombat + 1
      self.needsUpdate = true
      if self.inCombat or M.O["continueEncounterWhenDead"] and IsEncounterInProgress() then
        self.encounterInProgress = true
      else
        self.encounterInProgress = false
      end
      
      self.timeSinceLastUpdate = self.timeSinceLastUpdate - 1
    end
    
    if self.needsUpdate == true then
      self.needsUpdate = false
      self.updateDisplay()
    end
    
    if (self.COMlastStatusRequestSent >= 0) and (self.COMlastStatusRequestSent < (time() - 10)) then --wait 10 sec for reply then ask anyway
      self.COMlastStatusRequestSent = -1
      self:checkLogging()
    end
    
  end
  frame.updateDisplay = function()
    local l = frame.secondsInCombat
    local d = math.floor(l / 86400)
    l = l - 86400 * d
    local h = math.floor(l / 3600)
    l = l - 3600 * h
    local m = math.floor(l / 60)
    l = l - 60 * m
    local s = l
    
    local t
    
    if d > 0 then
      t = string.format("%dd %d:%02d:%02d", d, h, m, s)
    elseif h > 0 then
      t = string.format("%d:%02d:%02d", h, m, s)
    elseif m > 0 then
      t = string.format("%d:%02d", m, s)
    else
      t = s
    end
    if M.O["showCombatLogging"] then
      if frame.combatLogging then
        t = t.." |cff00ff00ON|r"
      else
        t = t.." |cffff0000OFF|r"
      end
    end
    if M.O["showYardsRunInCombat"] then
      t = t.." "..floor(frame.movement_done)
    end
    frame.text:SetText(t)
  end
  frame.combatLoggingEvent = function(msg)
    --isLogging = LoggingCombat([newState])
    frame.combatLogging = LoggingCombat() and true or false
    frame.updateDisplay()
  end
  hooksecurefunc(SlashCmdList, "COMBATLOG", frame.combatLoggingEvent)
  
  
  --Chat hyperlink stuff
  frame.MyOnHyperlinkShow = function(chatFrame, link, text, button)
    if link:sub(1, 15) ~= "FixRCombatTime:" then
      return
    end
    local info = ChatTypeInfo["SYSTEM"]
    if link:sub(16) == "LogYES" then
      LoggingCombat(true)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGENABLED, info.r, info.g, info.b, info.id)
    elseif link:sub(16,20) == "LogNO" then
      LoggingCombat(false)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGDISABLED, info.r, info.g, info.b, info.id)
      local instanceType = link:sub(21)
      if instanceType == "LFR" then
        setOptionBoolean("askIn_LFR", 0, true)
      elseif instanceType == "LFG" then
        setOptionBoolean("askIn_LFG", 0, true)
      elseif instanceType == "SCEN" then
        setOptionBoolean("askIn_Scenario", 0, true)
      elseif instanceType == "NRAID" then
        setOptionBoolean("askIn_Raid", 0, true)
      elseif instanceType == "NPARTY" then
        setOptionBoolean("askIn_Party", 0, true)
      end
    --[[
    elseif link:sub(16) == "LogNO" then
      LoggingCombat(false)
      DEFAULT_CHAT_FRAME:AddMessage(COMBATLOGDISABLED, info.r, info.g, info.b, info.id)
    --]]
    end
    frame.combatLoggingEvent()
  end
  hooksecurefunc("ChatFrame_OnHyperlinkShow", frame.MyOnHyperlinkShow)
  local old_SetHyperlink = ItemRefTooltip.SetHyperlink -- http://forums.wowace.com/showthread.php?t=20217
	function ItemRefTooltip:SetHyperlink(link, ...)
		if link:sub(1, 15) == "FixRCombatTime:" then
			return
		end
		return old_SetHyperlink(self, link, ...)
	end
  
  function frame:checkLogging()
    if not self.lastCheckLogging then self.lastCheckLogging = time()-10 end
    if self.lastCheckLogging > (time() - 10) then
      return
    else
      self.lastCheckLogging = time()
    end
    
    self.COMlastStatusRequestSent = -1
    
    if M.O["combatLoggingReminder"] then
      if not LoggingCombat() then
        local instanceType = getInstanceType()
        if instanceType then
          local typeString = instanceTypeString[instanceType] or "unknown"
          FixR:Echo("Not logging combat, start now? \124cffaaffaa\124HFixRCombatTime:LogYES\124h[yes]\124h", " or \124cffff8888\124HFixRCombatTime:LogNO"..instanceType.."\124h[never in "..typeString.."]\124h")
          confirmationFrame:showDialog(instanceType, typeString)
        end
      --else
      --  FixR:Echo("CombatTime: |cff00ff00Logging combat.")
      end
    end
  end
  
  function frame:COMrequestStatus()
    if (M.O["combatLoggingReminder"]) and (not LoggingCombat()) then
      local inInstance, _ = IsInInstance()
      if inInstance then
        local blnAsk = false
        local instanceType = getInstanceType()
        if (instanceType == "LFR") and (M.O["askIn_LFR"]) then --LFR
          blnAsk = true
        elseif (instanceType == "LFG") and (M.O["askIn_LFG"]) then --LFG
          blnAsk = true
        elseif (instanceType == "SCEN") and (M.O["askIn_Scenario"]) then --Scenario
          blnAsk = true
        elseif (instanceType == "NRAID") and (M.O["askIn_Raid"]) then --10/25 man
          blnAsk = true
        elseif (instanceType == "NPARTY") and (M.O["askIn_Party"]) then --5 man
          blnAsk = true
        end
        if blnAsk then
          self.COMlastStatusRequestSent = time()
          SendAddonMessage(self.COMprefix, "statusreq", self.COMchannel)
        end
      end
    end
  end
  
  instanceTypeString = {
    ["LFR"] = "LFR";
    ["LFG"] = "LFG";
    ["SCEN"] = "Scenario";
    ["NRAID"] = "normal Raid";
    ["NPARTY"] = "normal Party";
  }
  
  confirmationFrame = CreateFrame("Frame", "FixR_CombatTimeConfirmationFrame", UIParent)
  confirmationFrame:SetPoint("CENTER", UIParent, "CENTER")
  confirmationFrame:SetFrameStrata("DIALOG")
  confirmationFrame:SetSize(166, 70)
  
  confirmationFrame:SetMovable(true)
  confirmationFrame:EnableMouse(true)
  
  confirmationFrame:Hide()
  
  confirmationFrame.bgtexture = confirmationFrame:CreateTexture(nil, "BACKGROUND")
  confirmationFrame.bgtexture:SetAllPoints(confirmationFrame)
  confirmationFrame.bgtexture:SetTexture(0, 0, 0, 0.8)
  confirmationFrame.bgtexture:Show()
  
  confirmationFrame.text = confirmationFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  confirmationFrame.text:SetPoint("TOPLEFT", 4, -4)
  confirmationFrame.text:SetText("Start combat log?")
  confirmationFrame.text:SetTextColor(1, 1, 1, 1)
  
  confirmationFrame.dialogInstanceType = "UNKNOWN"
  confirmationFrame.dialogInstanceTypeString = "unknown"
  
  local newButton = function(parent, posSelf, posRelativeTo, posRelative, ox, oy, w, h, text)
    local retButton = CreateFrame("Button", nil, parent)
    retButton:SetPoint(posSelf, posRelativeTo, posRelative, ox, oy)
    retButton:SetFrameStrata("DIALOG")
    retButton:SetWidth(w)
    retButton:SetHeight(h)
    
    retButton:SetText(text)
    retButton:SetNormalFontObject("GameFontNormal")
    
    local ntex = retButton:CreateTexture()
    ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
    ntex:SetTexCoord(0, 0.625, 0, 0.6875)
    ntex:SetAllPoints()	
    retButton:SetNormalTexture(ntex)
    
    local htex = retButton:CreateTexture()
    htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    htex:SetTexCoord(0, 0.625, 0, 0.6875)
    htex:SetAllPoints()
    retButton:SetHighlightTexture(htex)
    
    local ptex = retButton:CreateTexture()
    ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
    ptex:SetTexCoord(0, 0.625, 0, 0.6875)
    ptex:SetAllPoints()
    retButton:SetPushedTexture(ptex)

    retButton:Show();
    
    return retButton
  end
  
  confirmationFrame.buttonsYES = newButton(confirmationFrame, "TOPLEFT", confirmationFrame, "TOPLEFT", 2, -22, 80, 22, "Yes")
  confirmationFrame.buttonsNO = newButton(confirmationFrame, "TOPLEFT", confirmationFrame.buttonsYES, "TOPRIGHT", 0, 0, 80, 22, "No")
  confirmationFrame.buttonsNEVER = newButton(confirmationFrame, "TOPLEFT", confirmationFrame.buttonsYES, "BOTTOMLEFT", 0, -2, 162, 22, "Never")
  
  confirmationFrame.buttonsYES:SetScript("OnClick", function(self, arg1)
    frame.MyOnHyperlinkShow(nil, "FixRCombatTime:LogYES")
    self:GetParent():Hide()
  end);
  confirmationFrame.buttonsNO:SetScript("OnClick", function(self, arg1)
    frame.MyOnHyperlinkShow(nil, "FixRCombatTime:LogNO")
    self:GetParent():Hide()
  end);
  confirmationFrame.buttonsNEVER:SetScript("OnClick", function(self, arg1)
    frame.MyOnHyperlinkShow(nil, "FixRCombatTime:LogNO"..(self:GetParent().dialogInstanceType))
    self:GetParent():Hide()
  end);
  
  confirmationFrame.showDialog = function(self, instanceType, instanceTypeString)
    self.dialogInstanceType = instanceType
    self.dialogInstanceTypeString = instanceTypeString
    confirmationFrame.buttonsNEVER:SetText("Never in "..instanceTypeString)
    self:Show()
  end
  --/run FixR_CombatTimeConfirmationFrame:showDialog("WHAT","no plan")
end

--[[
function frameEvents:UNIT_ABSORB_AMOUNT_CHANGED(unit)
  if unit == "player" then
    absorbAmount = UnitGetTotalAbsorbs("player")
    self.updateDisplay()
  end
end
--]]
function frameEvents:PLAYER_REGEN_DISABLED(...)
  if self.encounterInProgress == false then
    self.startEncounter()
  end
  self.inCombat = true
  self.updateDisplay()
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self.updateDisplay()
end
--function frameEvents:GROUP_ROSTER_UPDATE(...)
  --print("GROUP_ROSTER_UPDATE")
  --self:checkLogging()
--end
--[[
function frameEvents:PARTY_MEMBERS_CHANGED(...)
end
function frameEvents:RAID_ROSTER_UPDATE(...)
end
--]]
function frameEvents:CHAT_MSG_ADDON(prefix, msg, channel, from)
  if prefix == self.COMprefix then
    if (msg == "statusreq") and (from ~= self.COMPlayerName) then
      if LoggingCombat() then
        --disable answer for now, since it's pretty much redundant
        --SendAddonMessage(self.COMprefix, "LogON", "WHISPER", from)
      end
    elseif msg == "LogON" then
      self:checkLogging()
    end
  end
end
function frameEvents:ZONE_CHANGED()
  self:movement_resetMapScale()
end
function frameEvents:ZONE_CHANGED_INDOORS()
  self:movement_resetMapScale()
end
function frameEvents:ZONE_CHANGED_NEW_AREA()
  --self:checkLogging()
  --request status when entering an instance
  self:COMrequestStatus()
  self:movement_resetMapScale()
end

function DBMEventHandler:pull(event, mod, delay, synced)
  if M.O["useDBMPullDetection"] then
    --if frame.encounterInProgress == false then
      --let DBM:pull always reset the combat session
      frame.startEncounter()
    --end
    frame.updateDisplay()
  end
end



function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k);
  end
  if DBM and DBM.RegisterCallback then
    for k, v in pairs(DBMEventHandler) do
      DBM:RegisterCallback(k, v)
    end
  end
  frame:Show()
  
  if LoggingCombat() then
    frame.updateDisplay()
  elseif (self.O["autoLogAfterDisconnect"]) and (self.O["logoutTimestamp"] >= 0) then
    if self.O["logoutTimestamp"] >= (time() - 60*10) then
      frame.MyOnHyperlinkShow(nil, "FixRCombatTime:LogYES")
      frame.updateDisplay()
    else
      frame:COMrequestStatus()
    end
    self.O["logoutTimestamp"] = -1
  else
    --You know this one time, at login...
    frame:COMrequestStatus()
  end
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k);
    end
    frame:Hide()
  end
end

function M:Logout()
  if LoggingCombat() then
    self.O["logoutTimestamp"] = time()
  else
    self.O["logoutTimestamp"] = -1
  end
end

setOptionBoolean = function(optName, arg2, forceUpdateGUI)
  local bln, updateGUI = nil, true
  if arg2 == 1 then
    bln = true
    updateGUI = false
  elseif arg2 == 0 then
    bln = false
    updateGUI = false
  else --toggle
    bln = not (M.O[optName] or false)
  end
  M.O[optName] = bln
  FixR:Echo("CombatTime: "..optName..": "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  if updateGUI or forceUpdateGUI then
    FixR:SetModuleGUIOption(M, getOptionIndexByName(optName), bln)
  end
  return bln, updateGUI --return <new_state>, <gui_needed_update>
end

getInstanceType = function()
  local _, instanceType, difficultyID, _, _, _, _, _ = GetInstanceInfo()
  if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then --LFR/LFG
    if instanceType == "raid" then --LFR
      return "LFR"
    elseif instanceType == "party" then --LFG
      return "LFG"
    elseif instanceType == "scenario" then
      return "SCEN"
    end
    --LE_LFG_CATEGORY_FLEXRAID (difficultyID == 14)
  elseif IsInRaid() then --10/25 man
    return "NRAID"
  else --elseif IsInGroup() then --5 man
    return "NPARTY"
  end
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("CombatTime: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  elseif arg1 == "showCombatLogging" then
    setOptionBoolean(arg1, arg2)
    if frame then
      frame.updateDisplay()
    end
  elseif arg1 == "combatLoggingReminder" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_LFR" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_LFG" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Scenario" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Raid" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "askIn_Party" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "autoLogAfterDisconnect" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "continueEncounterWhenDead" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "useDBMPullDetection" then
    setOptionBoolean(arg1, arg2)
  elseif arg1 == "showYardsRunInCombat" then
    setOptionBoolean(arg1, arg2)
  end
end
