--[[ Vengeance ]]
-- partially from "infVengeance"

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    },
    {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    },
  };
}
local function getOptionIndexByName(name)
  if misc and misc["GUI"] then
    for i, v in ipairs(misc["GUI"]) do
      if v["opt"] == name then
        return i
      end
    end
  end
  return 0
end

local M = FixR:NewModule("Vengeance", DefaultO, misc)

local frame = nil
local frameEvents = {};
local setOptionBoolean
local currentVengeance = 0
local vengeanceId = GetSpellInfo(132365)
local frameFRMarker = nil
local frameFRMarkerEvents = {};

local UnitAttackPower, UnitStat, UnitPower, UnitHealth, UnitHealthMax = UnitAttackPower, UnitStat, UnitPower, UnitHealth, UnitHealthMax

local function initFrame()
  frame = CreateFrame("Frame", "FixR_VengeanceFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("LEFT")
  frame.text:SetText("Vengeance")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frameFRMarker = CreateFrame("Frame", "FixR_VengeanceFRMarker", UIParent)
  frameFRMarker.myparent = UIParent
  frameFRMarker:SetPoint("TOP", frameFRMarker.myparent, "TOPLEFT")
  frameFRMarker:SetFrameStrata("HIGH")
  frameFRMarker:SetSize(16, 16)
  if IsAddOnLoaded("PitBull4") then
    frameFRMarker:SetParent(PitBull4_Frames_player)
    frameFRMarker.myparent = PitBull4_Frames_player
    frameFRMarker:SetPoint("TOP", frameFRMarker.myparent, "TOPLEFT")
    frameFRMarker:SetFrameStrata("HIGH")
    frameFRMarker:SetSize(16, 16)
  end
  frameFRMarker:SetScript("OnEvent", function(self, event, ...)
    frameFRMarkerEvents[event](self, ...)
  end);
  frameFRMarker:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  
  frameFRMarker.bgtexture = frameFRMarker:CreateTexture(nil, "BACKGROUND")
  --frameFRMarker.bgtexture:SetAllPoints(frameFRMarker)
  --frameFRMarker.bgtexture:SetTexture(0, 0, 0, 0.8)
  frameFRMarker.bgtexture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_4")
  --frameFRMarker.bgtexture:SetSize(frameSize-4, frameSize-4)
  --frameFRMarker.bgtexture:SetPoint("TOPLEFT", 2, -2)
  frameFRMarker.bgtexture:SetAllPoints(frameFRMarker)
  frameFRMarker.bgtexture:Show()
  
  frameFRMarker.timeSinceLastUpdate = 0
  frameFRMarker.timerInterval = 0.25
  
  frameFRMarker.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    if (self.timeSinceLastUpdate >= self.timerInterval) then --in seconds
      self:updatePosition()
      
      self.timeSinceLastUpdate = 0
    end
  end
  
  frameFRMarker.updatePosition = function(self)
    --UnitPower("player")
    --((UnitHealth("player") + 0) / UnitHealthMax("player"))
    --PitBull4_Frames_player width=225
    --local base, posBuff, negBuff = UnitAttackPower("player");return base + posBuff + negBuff
    --max(2 * (AttackPower - Agility * 2), Stamina * 2.5)
    --select(2,UnitStat("player",2)) --agility
    --select(2,UnitStat("player",3)) --stamina
    
    local a,b,c=UnitAttackPower("player")
    local n=max(2*(a+b+c-select(2,UnitStat("player",2))*2),select(2,UnitStat("player",3))*2.5)
    local p=min(UnitPower("player"),60)
    n=(p/60)*n
    local xo=((UnitHealth("player")+n)/UnitHealthMax("player"))*225
    self:SetPoint("TOP", self.myparent, "TOPLEFT", xo, 0)
  end
  
  frameFRMarker.updateShapeshift = function(self)
    local nStance = GetShapeshiftForm()
    if nStance == 1 then --Bear Form
      --self:Show()
      --disable for now
      self:Hide()
    else
      self:Hide()
    end
  end
  
  frameFRMarker:updateShapeshift()
end

function frameEvents:UNIT_AURA(unitId)
  if unitId ~= "player" then return end
  local name, _, _, _, _, _, _, _, _, _, _, _, _, _, amount, _ = UnitBuff(unitId, vengeanceId)
  if name then
    if currentVengeance ~= amount then
      currentVengeance = amount
      self.text:SetText(amount)
      
      frameFRMarker:updatePosition()
    end
  else
    if currentVengeance > 0 then
      currentVengeance = 0
      self.text:SetText("0")
      frameFRMarker:updatePosition()
    end
  end
end
function frameFRMarkerEvents:ADDON_LOADED(arg1)
  if arg1 == "PitBull4" then
    frameFRMarker:UnregisterEvent("ADDON_LOADED")
    
    frameFRMarker:SetParent(PitBull4_Frames_player)
    frameFRMarker.myparent = PitBull4_Frames_player
    frameFRMarker:SetPoint("TOP", frameFRMarker.myparent, "TOPLEFT")
    frameFRMarker:SetFrameStrata("HIGH")
    frameFRMarker:SetSize(16, 16)
  end
end
function frameFRMarkerEvents:UPDATE_SHAPESHIFT_FORM(...)
  frameFRMarker:updateShapeshift()
end

function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k)
  end
  frame:Show()
  
  for k, v in pairs(frameFRMarkerEvents) do
    frameFRMarker:RegisterEvent(k)
  end
  if IsAddOnLoaded("PitBull4") then
    frameFRMarker:UnregisterEvent("ADDON_LOADED")
  end
  frameFRMarker:Show()
  frameFRMarker:updateShapeshift()
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k)
    end
    frame:Hide()
    for k, v in pairs(frameFRMarkerEvents) do
      frameFRMarker:UnregisterEvent(k)
    end
    frameFRMarker:Hide()
  end
end

setOptionBoolean = function(optName, arg2, forceUpdateGUI)
  local bln, updateGUI = nil, true
  if arg2 == 1 then
    bln = true
    updateGUI = false
  elseif arg2 == 0 then
    bln = false
    updateGUI = false
  else --toggle
    bln = not (M.O[optName] or false)
  end
  M.O[optName] = bln
  FixR:Echo("Vengeance: "..optName..": "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  if updateGUI or forceUpdateGUI then
    FixR:SetModuleGUIOption(M, getOptionIndexByName(optName), bln)
  end
  return bln, updateGUI --return <new_state>, <gui_needed_update>
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("Vengeance: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  end
end
